library(matlab)
library(mvtnorm)
library(ergm)
library(igraph)

set.seed(1)

logsumexp <- function(x){
  xmax = which.max(x)
  if (sum(xmax)==0)
  {
    -Inf
  }
  else
  {
    result = log1p(sum(exp(x[-xmax]-x[xmax])))+x[xmax]
    if (is.nan(result))
    {
      -Inf
    }
    else
    {
      result
    }
  }
}

stratified_resample_u<-function(log_weights,u)
{
  P = length(log_weights)
  W = exp(log_weights)
  cw = cumsum(W / sum(W))
  n = length(W)
  u = u/n
  v = u + (0:(n-1))/n
  
  j = 1
  indices = matrix(0,n)
  for (i in 1:n)
  {
    while(cw[j] < v[i])
    {
      j = j+1
    }
    indices[i] = j
  }
  
  indices
}

score_u <- function(epsilon,current_error,alpha,num_imp_u,resample_u,current_u)
{
  which_nonzero = which(current_error<epsilon)
  log_weights = matrix(-Inf,num_imp_u)
  log_weights[which_nonzero] = -log(num_imp_u)
  if (all(log_weights==-Inf))
  {
    1-alpha*num_imp_u
  } else {
    indices = stratified_resample_u(log_weights,resample_u)
    current_u = current_u[,indices]
    length(unique(current_u[1,]))-alpha*num_imp_u
  }
}

bisection_u <- function(range,current_error,alpha,num_imp_u,resample_u,current_u)
{
  current_bisect_epsilon = range[2]
  bisect_size = (range[2] - range[1])/2
  direction = -1
  
  old_score = score_u(current_bisect_epsilon,current_error,alpha,num_imp_u,resample_u,current_u)
  
  for (i in 1:100)
  {
    new_bisect_epsilon = current_bisect_epsilon + direction*bisect_size
    bisect_size = bisect_size/2
    the_score = score_u(new_bisect_epsilon,current_error,alpha,num_imp_u,resample_u,current_u)
    direction = -sign(the_score)
    
    current_bisect_epsilon = new_bisect_epsilon
    
    if (the_score==0)
    {
      break
    }
    
  }
  
  current_bisect_epsilon
}

da_score <- function(da_epsilon,proposed_error,old_error,num_kept)
{
  length(which((proposed_error<da_epsilon)&(old_error<da_epsilon)==TRUE))-num_kept
}

da_bisection <- function(range,proposed_error,old_error,num_kept)
{
  current_bisect_epsilon = range[2]
  bisect_size = (range[2] - range[1])/2
  direction = -1

  old_score = da_score(current_bisect_epsilon,proposed_error,old_error,num_kept)
  
  for (i in 1:100)
  {
    new_bisect_epsilon = current_bisect_epsilon + direction*bisect_size
    bisect_size = bisect_size/2
    the_score = da_score(new_bisect_epsilon,proposed_error,old_error,num_kept)
    direction = -sign(the_score)
    
    current_bisect_epsilon = new_bisect_epsilon
    
    if (the_score==0)
    {
      break
    }
    
  }
  
  current_bisect_epsilon
}

dolphin_igraph <- read.graph("dolphins.gml", format = "gml")
dm = as.matrix(get.adjacency(dolphin_igraph))
dolphin = network(dm,directed=FALSE)
y = dolphin

num_nodes = length(y$val)

ysum = summary(y~edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE))


ninternal = 15000
reduced_ninternal = 1500

num_kept = 100

num_imp_u = 300
alpha = num_kept/num_imp_u
epsilon = 1

epsilons = c(165,156,120,72,42,24,14,10,8,6,4,2,0)

prior_sd = sqrt(30)

total_sims <<- 0

complete_graph = simulate(network(num_nodes,directed=FALSE) ~ edges, coef=c(1000000), nsim=1, control=control.simulate(MCMC.burnin=10000,MCMC.interval=1))
empty_graph = simulate(network(num_nodes,directed=FALSE) ~ edges, coef=c(-1000000), nsim=1, control=control.simulate(MCMC.burnin=10000,MCMC.interval=1))
current_epsilon = sqrt(sum((summary(complete_graph~edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE)) - summary(empty_graph~edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE)))^2))

current_u = matrix(0,4,num_imp_u)
temp_u = matrix(0,4,num_kept)
temp_u[1,] = rnorm(num_kept,0,prior_sd)
temp_u[2,] = rnorm(num_kept,0,prior_sd)
temp_u[3,] = rnorm(num_kept,0,prior_sd)
temp_u[4,] = rnorm(num_kept,0,prior_sd)
current_u = repmat(temp_u,1,num_imp_u/num_kept)

prop_sd = sd(current_u)

current_reduced_x = rep(complete_graph,num_imp_u)
temp_reduced_x = rep(complete_graph,num_kept)
current_reduced_latent_x = rep(complete_graph,num_imp_u)
temp_reduced_latent_x = rep(complete_graph,num_kept)

for (g in 1:(num_kept))
{
  current_x_latent = simulate(network(num_nodes,directed=FALSE) ~ edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE), coef=temp_u[1:3,g], nsim=1, control=control.simulate(MCMC.burnin=reduced_ninternal-1,MCMC.interval=1))
  
  temp_reduced_latent_x[[g]] = current_x_latent
  edges = matrix(2*as.matrix.network(current_x_latent)-1,num_nodes*num_nodes)
  new_matrix = (matrix(as.matrix(simulate_ising_observations(as.vector(edges),temp_u[4,g])),num_nodes,num_nodes)+1)/2
  for(i in 1:nrow(new_matrix)) {for(j in 1:i) {new_matrix[i,j]=new_matrix[j,i] }}
  temp_reduced_x[[g]] = network(new_matrix,directed=FALSE)
  total_sims <<- total_sims + (reduced_ninternal)
}

for (h in 1:(num_imp_u/num_kept))
{
  for (g in 1:(num_kept))
  {
    current_reduced_x[[(h-1)*num_kept+g]] = temp_reduced_x[[g]]
  }
}

current_x = rep(complete_graph,num_imp_u)
temp_x = rep(complete_graph,num_kept)

for (g in 1:(num_kept))
{
  initial = temp_reduced_latent_x[[g]]
  current_x_latent = simulate(network(num_nodes,directed=FALSE) ~ edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE), coef=temp_u[1:3,g], nsim=1, control=control.simulate(MCMC.burnin=ninternal-reduced_ninternal-1,MCMC.interval=1), basis=initial)
  
  edges = matrix(2*as.matrix.network(current_x_latent)-1,num_nodes*num_nodes)
  new_matrix = (matrix(as.matrix(simulate_ising_observations(as.vector(edges),temp_u[4,g])),num_nodes,num_nodes)+1)/2
  for(i in 1:nrow(new_matrix)) {for(j in 1:i) {new_matrix[i,j]=new_matrix[j,i] }}
  temp_x[[g]] = network(new_matrix,directed=FALSE)
  total_sims <<- total_sims + (ninternal-reduced_ninternal)
  error = sqrt(sum((summary(temp_x[[g]]~edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE))-ysum)^2))
}

for (h in 1:(num_imp_u/num_kept))
{
  for (g in 1:(num_kept))
  {
    current_x[[(h-1)*num_kept+g]] = temp_x[[g]]
  }
}

current_error = matrix(0,(num_imp_u))
for (g in 1:(num_imp_u))
{
  current_error[g] = sqrt(sum((summary(current_x[[g]]~edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE))-ysum)^2))
  
}

current_w = matrix(-log(num_imp_u),num_imp_u)

first = TRUE

while (current_epsilon>=epsilon)
{
  old_epsilon = current_epsilon
  resample_u = runif(num_imp_u,0,1)
  
  current_epsilon = bisection_u(c(0,current_epsilon),current_error,alpha,num_imp_u,resample_u,current_u)#epsilons[e]#
  
  for (g in 1:num_imp_u)
  {
    if (current_error[g]<current_epsilon)
    {
      current_w[g] = -log(num_imp_u)
    }
    else
    {
      current_w[g] = -Inf
    }
  }
  
  print("Epsilon:")
  print(current_epsilon)
  
  if (  (current_epsilon<epsilon) ) #|| (old_epsilon == current_epsilon) )
  {
    for (g in 1:num_imp_u)
    {
      if (current_error[g]<epsilon)
      {
        current_w[g] = -log(num_imp_u)
      }
      else
      {
        current_w[g] = -Inf
      }
    }
    
    indices = stratified_resample_u(current_w,resample_u)
    current_u = current_u[,indices]
    current_error = current_error[indices]
    current_w = matrix(-log(num_imp_u),num_imp_u)
    
    old_current_x = current_x
    old_current_reduced_x = current_reduced_x
    for (i in 1:length(indices))
    {
      current_x[[i]] = old_current_x[[indices[i]]]
      current_reduced_x[[i]] = old_current_reduced_x[[indices[i]]]
    }
    
    plot(current_u[1,],current_u[2,])
    
    print(total_sims)
    print(length(unique(current_u[1,])))
    
    print("Number:")
    print(number_accepted)
    
    line = toString(number_accepted)
    write(line,file="past_stage2_ergm.txt",append=TRUE)
    
    line = toString(total_sims)
    write(line,file="total_sims_ergm.txt",append=TRUE)
    
    line = toString(current_epsilon)
    write(line,file="epsilon_ergm.txt",append=TRUE)
    
    line = toString(da_epsilon)
    write(line,file="da_epsilon_ergm.txt",append=TRUE)
    
    line = toString(length(unique(current_u[1,])))
    write(line,file="unique_ergm.txt",append=TRUE)
    
    break
  }
  
  indices = stratified_resample_u(current_w,resample_u)
  
  current_u = current_u[,indices]
  current_error = current_error[indices]
  current_w = matrix(-log(num_imp_u),num_imp_u)
  
  old_current_x = current_x
  old_current_reduced_x = current_reduced_x
  for (i in 1:length(indices))
  {
    current_x[[i]] = old_current_x[[indices[i]]]
    current_reduced_x[[i]] = old_current_reduced_x[[indices[i]]]
  }
  
  print(length(unique(current_u[1,])))
  
  plot(current_u[1,],current_u[2,])
  
  prop_cov = cov(t(current_u))
  
  all_logu = log(runif(num_imp_u))
  all_proposed_u = matrix(0,4,num_imp_u)
  all_logratio = matrix(0,num_imp_u)
  old_reduced_x = current_reduced_x
  old_reduced_error = matrix(Inf,num_imp_u)
  
  for (g in 1:num_imp_u)
  {
    all_proposed_u[,g] = rmvnorm(1,current_u[,g],prop_cov)
    all_logratio[g] = dnorm(all_proposed_u[1,g],0,prior_sd,log=TRUE) + dnorm(all_proposed_u[2,g],0,prior_sd,log=TRUE) + dnorm(all_proposed_u[3,g],0,prior_sd,log=TRUE) + dnorm(all_proposed_u[4,g],0,prior_sd,log=TRUE) - dnorm(current_u[1,g],0,prior_sd,log=TRUE) - dnorm(current_u[2,g],0,prior_sd,log=TRUE) - dnorm(current_u[3,g],0,prior_sd,log=TRUE) - dnorm(current_u[4,g],0,prior_sd,log=TRUE)
    old_reduced_error[g] = sqrt(sum((summary(old_reduced_x[[g]]~edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE))-ysum)^2))
  }
  
  which_accept = which(all_logu<all_logratio)
  
  print("Num past prior")
  print(length(which_accept))
  
  line = toString(length(which_accept))
  write(line,file="past_prior_ergm.txt",append=TRUE)
  
  reduced_proposed_x = current_reduced_x
  reduced_proposed_latent_x = current_reduced_x
  reduced_proposed_error = matrix(Inf,num_imp_u)
  
  for (g in 1:length(which_accept))
  {
    proposed_x_latent = simulate(network(num_nodes,directed=FALSE) ~ edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE), coef=all_proposed_u[1:3,which_accept[g]], nsim=1, control=control.simulate(MCMC.burnin=reduced_ninternal-1,MCMC.interval=1))
    
    reduced_proposed_latent_x[[which_accept[g]]] = proposed_x_latent
    
    edges = matrix(2*as.matrix.network(proposed_x_latent)-1,num_nodes*num_nodes)
    new_matrix = (matrix(as.matrix(simulate_ising_observations(as.vector(edges),all_proposed_u[4,which_accept[g]])),num_nodes,num_nodes)+1)/2
    for(i in 1:nrow(new_matrix)) {for(j in 1:i) {new_matrix[i,j]=new_matrix[j,i] }}
    reduced_proposed_x[[which_accept[g]]] = network(new_matrix,directed=FALSE)
    
    total_sims <<- total_sims + (reduced_ninternal)
    reduced_proposed_error[which_accept[g]] = sqrt(sum((summary(reduced_proposed_x[[which_accept[g]]]~edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE))-ysum)^2))
    
  }
  
  da_epsilon = max(max(reduced_proposed_error[which_accept]),max(old_reduced_error[which_accept]))
  current_score = da_score(da_epsilon,reduced_proposed_error[which_accept],old_reduced_error[which_accept],num_kept)
  
  if (current_score>0)
  {
    da_epsilon = da_bisection(c(0,da_epsilon),reduced_proposed_error[which_accept],old_reduced_error[which_accept],num_kept)
  }
  
  which_accept = which( (reduced_proposed_error<=da_epsilon) & (old_reduced_error<=da_epsilon) )
  
  print("Num past stage 1")
  print(length(which_accept))
  
  line = toString(length(which_accept))
  write(line,file="past_stage1_ergm.txt",append=TRUE)
  
  plot(current_u[1,which_accept],current_u[2,which_accept])
  
  number_accepted = 0
  
  for (g in 1:length(which_accept))
  {
    initial = reduced_proposed_latent_x[[which_accept[g]]]
    proposed_x_latent = simulate(network(num_nodes,directed=FALSE) ~ edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE), coef=all_proposed_u[1:3,which_accept[g]], nsim=1, control=control.simulate(MCMC.burnin=ninternal-reduced_ninternal-1,MCMC.interval=1), basis=initial)
    
    edges = matrix(2*as.matrix.network(proposed_x_latent)-1,num_nodes*num_nodes)
    new_matrix = (matrix(as.matrix(simulate_ising_observations(as.vector(edges),all_proposed_u[4,which_accept[g]])),num_nodes,num_nodes)+1)/2
    for(i in 1:nrow(new_matrix)) {for(j in 1:i) {new_matrix[i,j]=new_matrix[j,i] }}
    proposed_x = network(new_matrix,directed=FALSE)
    
    total_sims <<- total_sims + (ninternal-reduced_ninternal)
    
    proposed_error = sqrt(sum((summary(proposed_x~edges+gwdegree(0.8,fixed=TRUE)+gwesp(0.8,fixed=TRUE))-ysum)^2))
    
    if (proposed_error<current_epsilon)
    {
      current_u[,which_accept[g]] = all_proposed_u[,which_accept[g]]
      current_x[[which_accept[g]]] = proposed_x
      current_reduced_x[[which_accept[g]]] = reduced_proposed_x[[which_accept[g]]]
      current_error[which_accept[g]] = proposed_error
      number_accepted = number_accepted + 1
    }
  }
  
  print("Number:")
  print(number_accepted)
  
  line = toString(number_accepted)
  write(line,file="past_stage2_ergm.txt",append=TRUE)
  
  line = toString(total_sims)
  write(line,file="total_sims_ergm.txt",append=TRUE)
  
  line = toString(current_epsilon)
  write(line,file="epsilon_ergm.txt",append=TRUE)
  
  line = toString(da_epsilon)
  write(line,file="da_epsilon_ergm.txt",append=TRUE)
  
  line = toString(length(unique(current_u[1,])))
  write(line,file="unique_ergm.txt",append=TRUE)
}
