library(matlab)
library(mvtnorm)

logsumexp <- function(x){
  xmax = which.max(x)
  if (sum(xmax)==0)
  {
    -Inf
  }
  else
  {
    result = log1p(sum(exp(x[-xmax]-x[xmax])))+x[xmax]
    if (is.nan(result))
    {
      -Inf
    }
    else
    {
      result
    }
  }
}

stratified_resample_u<-function(log_weights,u)
{
  P = length(log_weights)
  W = exp(log_weights)
  cw = cumsum(W / sum(W))
  n = length(W)
  u = u/n
  v = u + (0:(n-1))/n
  
  j = 1
  indices = matrix(0,n)
  for (i in 1:n)
  {
    while(cw[j] < v[i])
    {
      j = j+1
    }
    indices[i] = j
  }
  
  indices
}

score_u <- function(epsilon,current_error,alpha,num_imp_u,resample_u,current_u)
{
  which_nonzero = which(current_error<epsilon)
  log_weights = matrix(-Inf,num_imp_u)
  log_weights[which_nonzero] = -log(num_imp_u)
  if (all(log_weights==-Inf))
  {
    1-alpha*num_imp_u
  } else {
    indices = stratified_resample_u(log_weights,resample_u)
    current_u = current_u[,indices]
    length(unique(current_u[1,]))-alpha*num_imp_u
  }
}

bisection_u <- function(range,current_error,alpha,num_imp_u,resample_u,current_u)
{
  current_bisect_epsilon = range[2]
  bisect_size = (range[2] - range[1])/2
  direction = -1
  
  old_score = score_u(current_bisect_epsilon,current_error,alpha,num_imp_u,resample_u,current_u)
  
  for (i in 1:100)
  {
    new_bisect_epsilon = current_bisect_epsilon + direction*bisect_size
    bisect_size = bisect_size/2
    the_score = score_u(new_bisect_epsilon,current_error,alpha,num_imp_u,resample_u,current_u)
    direction = -sign(the_score)
    
    current_bisect_epsilon = new_bisect_epsilon
    
    if (the_score==0)
    {
      break
    }
    
  }
  
  current_bisect_epsilon
}

da_score <- function(da_epsilon,proposed_error,old_error,num_kept)
{
  length(which((proposed_error<da_epsilon)&(old_error<da_epsilon)==TRUE))-num_kept
}

da_bisection <- function(range,proposed_error,old_error,num_kept)
{
  current_bisect_epsilon = range[2]
  bisect_size = (range[2] - range[1])/2
  direction = -1

  old_score = da_score(current_bisect_epsilon,proposed_error,old_error,num_kept)
  
  for (i in 1:100)
  {
    new_bisect_epsilon = current_bisect_epsilon + direction*bisect_size
    bisect_size = bisect_size/2
    the_score = da_score(new_bisect_epsilon,proposed_error,old_error,num_kept)
    direction = -sign(the_score)
    
    current_bisect_epsilon = new_bisect_epsilon
    
    if (the_score==0)
    {
      break
    }

  }
  
  current_bisect_epsilon
}

logsumexp2elvec <- function(x){
  #latter_max = which(x[,1]<x[,2])
  #former_max = which(x[,1]<x[,2])
  maxRows = apply(x,1,max)
  log(rowSums(exp(x - repmat(maxRows,1,2)))) + maxRows
}

get_edges <- function(dimension)
{
  if (dimension<2)
    browser()#error
  edgesA = matrix(0,2*dimension*dimension-2*dimension,2)
  counter = 1
  from_edge_counter = 1
  to_edge_counter = 2
  for (i in 1:dimension)
  {
    for (j in 1:dimension)
    {
      if (j!=dimension)
      {
        edgesA[counter,1] = from_edge_counter
        edgesA[counter,2] = to_edge_counter
        from_edge_counter = from_edge_counter + 1
        to_edge_counter = to_edge_counter + 1
        counter = counter + 1
      }
      else
      {
        from_edge_counter = from_edge_counter + 1
        to_edge_counter = to_edge_counter + 1
      }
    }
  }
  
  from_edge_counter = 1
  to_edge_counter = dimension+1
  
  for (i in 1:dimension)
  {
    if (i!=dimension)
    {
      for (j in 1:dimension)
      {
        edgesA[counter,1] = from_edge_counter
        edgesA[counter,2] = to_edge_counter
        from_edge_counter = from_edge_counter + 1
        to_edge_counter = to_edge_counter + 1
        counter = counter + 1
      }
    }
  }
  
  return(edgesA)
}

set_up_ising_chess <- function(dimension)
{
  edgesA = get_edges(dimension)
  
  corners1 = matrix(0,1)
  corners2 = matrix(0,1)
  sides1 = matrix(0,1)
  sides2 = matrix(0,1)
  middles1 = matrix(0,1)
  middles2 = matrix(0,1)
  
  for (h in 1:(dimension*dimension))
  {
    #print(h)
    current_h = c(ceiling(h/dimension),h %% dimension)
    if (current_h[2]==0)
    {
      current_h[2] = dimension
    }
    index = which(edgesA==h,arr.ind=TRUE)
    
    if (nrow(index)==2)
    {
      if ((sum(current_h) %% 2)!=0)
        corners2 = append(corners2,h)
      else{
        corners1 = append(corners1,h)
      }
    }
    if (nrow(index)==3)
    {
      if ((sum(current_h) %% 2)!=0)
        sides2 = append(sides2,h)
      else{
        sides1 = append(sides1,h)
      }
    }
    if (nrow(index)==4)
    {
      if ((sum(current_h) %% 2)!=0)
        middles2 = append(middles2,h)
      else{
        middles1 = append(middles1,h)
      }
    }
  }
  
  corners1 = corners1[-1]
  corners2 = corners2[-1]
  sides1 = sides1[-1]
  sides2 = sides2[-1]
  middles1 = middles1[-1]
  middles2 = middles2[-1]
  
  corner_neighbours1 = matrix(0,length(corners1),2)
  for (p in 1:length(corners1))
  {
    index = which(edgesA==corners1[p],arr.ind=TRUE)
    if (length(index)>0)
    {
      for (k in 1:nrow(index))
      {
        corner_neighbours1[p,k] = edgesA[index[k,1],((-(2*index[k,2]-3))+3)/2]
      }
    }
  }
  
  corner_neighbours2 = matrix(0,length(corners2),2)
  for (p in 1:length(corners2))
  {
    index = which(edgesA==corners2[p],arr.ind=TRUE)
    if (length(index)>0)
    {
      for (k in 1:nrow(index))
      {
        corner_neighbours2[p,k] = edgesA[index[k,1],((-(2*index[k,2]-3))+3)/2]
      }
    }
  }
  
  side_neighbours1 = matrix(0,length(sides1),3)
  for (p in 1:length(sides1))
  {
    index = which(edgesA==sides1[p],arr.ind=TRUE)
    if (length(index)>0)
    {
      for (k in 1:nrow(index))
      {
        side_neighbours1[p,k] = edgesA[index[k,1],((-(2*index[k,2]-3))+3)/2]
      }
    }
  }
  
  side_neighbours2 = matrix(0,length(sides2),3)
  for (p in 1:length(sides2))
  {
    index = which(edgesA==sides2[p],arr.ind=TRUE)
    if (length(index)>0)
    {
      for (k in 1:nrow(index))
      {
        side_neighbours2[p,k] = edgesA[index[k,1],((-(2*index[k,2]-3))+3)/2]
      }
    }
  }
  
  middle_neighbours1 = matrix(0,length(middles1),4)
  for (p in 1:length(middles1))
  {
    index = which(edgesA==middles1[p],arr.ind=TRUE)
    if (length(index)>0)
    {
      for (k in 1:nrow(index))
      {
        middle_neighbours1[p,k] = edgesA[index[k,1],((-(2*index[k,2]-3))+3)/2]
      }
    }
  }
  
  middle_neighbours2 = matrix(0,length(middles2),4)
  for (p in 1:length(middles2))
  {
    index = which(edgesA==middles2[p],arr.ind=TRUE)
    if (length(index)>0)
    {
      for (k in 1:nrow(index))
      {
        middle_neighbours2[p,k] = edgesA[index[k,1],((-(2*index[k,2]-3))+3)/2]
      }
    }
  }
  
  x = list(corners1,corners2,sides1,sides2,middles1,middles2,corner_neighbours1,corner_neighbours2,side_neighbours1,side_neighbours2,middle_neighbours1,middle_neighbours2,edgesA)
  return(x)
}

find_statistic <- function(input,edges)
{
  sum(input[edges[,1]]==input[edges[,2]]) - sum(input[edges[,1]]!=input[edges[,2]])
}

simulate_ising_chess <- function(npoints,burnin,dimension,param,initial,ising_data)
{
  corners1 = ising_data[[1]]
  corners2 = ising_data[[2]]
  sides1 = ising_data[[3]]
  sides2 = ising_data[[4]]
  middles1 = ising_data[[5]]
  middles2 = ising_data[[6]]
  corner_neighbours1 = ising_data[[7]]
  corner_neighbours2 = ising_data[[8]]
  side_neighbours1 = ising_data[[9]]
  side_neighbours2 = ising_data[[10]]
  middle_neighbours1 = ising_data[[11]]
  middle_neighbours2 = ising_data[[12]]
  
  current_grid = initial
  all_grids = matrix(0,npoints,dimension*dimension)
  
  ninternal = burnin
  
  for (i in 1:(npoints+ninternal))
  {
    #print(i)
    
    prob1 = apply(param*kronecker(matrix(1,length(corners1),2),-1)*matrix(current_grid[matrix(corner_neighbours1,length(corners1)*2)],length(corners1),2),1,sum)
    prob2 = -prob1#apply(param*kronecker(matrix(1,length(corners1),2),1)*matrix(current_grid[matrix(corner_neighbours,length(corners1)*2)],length(corners1),2),1,sum)
    prob_to_use = exp(prob2 - logsumexp2elvec(cbind(prob1,prob2)))
    current_grid[corners1] = 2*as.integer(runif(length(corners1))<prob_to_use)-1
    
    if (length(side_neighbours1)>0)
    {
      prob1 = apply(param*kronecker(matrix(1,length(sides1),3),-1)*matrix(current_grid[matrix(side_neighbours1,length(sides1)*3)],length(sides1),3),1,sum)
      prob2 = -prob1#apply(param*kronecker(matrix(1,length(sides1),3),1)*matrix(current_grid[matrix(side_neighbours1,length(sides1)*3)],length(sides1),3),1,sum)
      prob_to_use = exp(prob2 - logsumexp2elvec(cbind(prob1,prob2)))
      current_grid[sides1] = 2*as.integer(runif(length(sides1))<prob_to_use)-1
    }
    
    if (length(middle_neighbours1)>0)
    {
      prob1 = apply(param*kronecker(matrix(1,length(middles1),4),-1)*matrix(current_grid[matrix(middle_neighbours1,length(middles1)*4)],length(middles1),4),1,sum)
      prob2 = -prob1#apply(param*kronecker(matrix(1,length(middles1),4),1)*matrix(current_grid[matrix(middle_neighbours1,length(middles1)*4)],length(middles1),4),1,sum)
      prob_to_use = exp(prob2 - logsumexp2elvec(cbind(prob1,prob2)))
      current_grid[middles1] = 2*as.integer(runif(length(middles1))<prob_to_use)-1
    }
    
    prob1 = apply(param*kronecker(matrix(1,length(corners2),2),-1)*matrix(current_grid[matrix(corner_neighbours2,length(corners2)*2)],length(corners2),2),1,sum)
    prob2 = -prob1#apply(param*kronecker(matrix(1,length(corners2),2),1)*matrix(current_grid[matrix(corner_neighbours2,length(corners2)*2)],length(corners2),2),1,sum)
    prob_to_use = exp(prob2 - logsumexp2elvec(cbind(prob1,prob2)))
    current_grid[corners2] = 2*as.integer(runif(length(corners2))<prob_to_use)-1
    
    if (length(side_neighbours2)>0)
    {
      prob1 = apply(param*kronecker(matrix(1,length(sides2),3),-1)*matrix(current_grid[matrix(side_neighbours2,length(sides2)*3)],length(sides2),3),1,sum)
      prob2 = -prob1#apply(param*kronecker(matrix(1,length(sides2),3),1)*matrix(current_grid[matrix(side_neighbours2,length(sides2)*3)],length(sides2),3),1,sum)
      prob_to_use = exp(prob2 - logsumexp2elvec(cbind(prob1,prob2)))
      current_grid[sides2] = 2*as.integer(runif(length(sides2))<prob_to_use)-1
    }
    
    if (length(middle_neighbours2)>0)
    {
      prob1 = apply(param*kronecker(matrix(1,length(middles2),4),-1)*matrix(current_grid[matrix(middle_neighbours2,length(middles2)*4)],length(middles2),4),1,sum)
      prob2 = -prob1#apply(param*kronecker(matrix(1,length(middles2),4),1)*matrix(current_grid[matrix(middle_neighbours2,length(middles2)*4)],length(middles2),4),1,sum)
      prob_to_use = exp(prob2 - logsumexp2elvec(cbind(prob1,prob2)))
      current_grid[middles2] = 2*as.integer(runif(length(middles2))<prob_to_use)-1
    }
    
    if (i>ninternal)
    {
      all_grids[i-ninternal,] = current_grid
    }
  }
  
  all_grids
}

simulate_ising_observations <- function(current_grid,param)
{
  #browser()
  prob2 = param*current_grid
  prob1 = -prob2
  if (is.vector(prob1))
    prob_to_use = exp(prob2 - logsumexp2elvec(cbind(prob1,prob2)))
  else
    prob_to_use = exp(prob2 - logsumexp2elvec(cbind(t(prob1),t(prob2))))
  observations = (2*as.integer(runif(length(current_grid))<prob_to_use)-1)
}


eps_store = c()
unique_before_store = c()
unique_store = c()
total_sims_store = c()
num_past_prior_store = c()
num_past_stage1_store = c()
num_accepted_store = c()
da_eps_store = c()
e1_store = c()
e2_store = c()
e3_store = c()
v1_store = c()
v2_store = c()
v3_store = c()

dimension = 10
ising_data = set_up_ising_chess(dimension)
edges = ising_data[[13]]

reduced_dimension = dimension
reduced_ising_data = set_up_ising_chess(reduced_dimension)
reduced_edges = reduced_ising_data[[13]]

y = as.matrix( read.table("~/code/da_ising/test_data_10_10_10_singleit.txt"))

ysquare = matrix(y,dimension,dimension)

reduced_ysquare = ysquare[1:reduced_dimension,1:reduced_dimension]

y = matrix(ysquare,1,dimension*dimension)

ysum = find_statistic(y,edges)

reduced_y = matrix(reduced_ysquare,1,reduced_dimension*reduced_dimension)
reduced_ysum = find_statistic(reduced_y,reduced_edges)

# ninternal = 1000
# reduced_ninternal = 5
# num_kept = 1000
# num_imp_u = 5000
# alpha = num_kept/num_imp_u
# epsilon = 1

prior_sd = 5

total_sims <<- 0

current_epsilon = 2*abs(find_statistic(matrix(1,dimension*dimension),edges))

current_u = matrix(0,2,num_imp_u)
temp_u = matrix(0,2,num_kept)
temp_u[1,] = rnorm(num_kept,0,prior_sd)
temp_u[2,] = rnorm(num_kept,0,prior_sd)
current_u = repmat(temp_u,1,num_imp_u/num_kept)

prop_sd = sd(current_u)

current_reduced_x = matrix(0,num_imp_u,reduced_dimension*reduced_dimension)
temp_reduced_x = matrix(0,num_kept,reduced_dimension*reduced_dimension)
current_reduced_latent_x = matrix(0,num_imp_u,reduced_dimension*reduced_dimension)
temp_reduced_latent_x = matrix(0,num_kept,reduced_dimension*reduced_dimension)
for (g in 1:(num_kept))
{
  current_x_latent = simulate_ising_chess(1,reduced_ninternal-1,reduced_dimension,temp_u[1,g],matrix(1,reduced_dimension*reduced_dimension),reduced_ising_data)
  temp_reduced_latent_x[g,] = current_x_latent
  temp_reduced_x[g,] = simulate_ising_observations(current_x_latent,temp_u[2,g])
  total_sims <<- total_sims + (reduced_ninternal)*dimension*dimension
}

for (h in 1:(num_imp_u/num_kept))
{
  for (g in 1:(num_kept))
  {
    current_reduced_x[(h-1)*num_kept+g,] = temp_reduced_x[g,]
  }
}

current_x = matrix(0,num_imp_u,dimension*dimension)
temp_x = matrix(0,num_kept,dimension*dimension)

for (g in 1:(num_kept))
{
  initial = matrix(repmat(matrix(temp_reduced_latent_x[g,],reduced_dimension,reduced_dimension),dimension/reduced_dimension,dimension/reduced_dimension),dimension*dimension)
  current_x_latent = simulate_ising_chess(1,ninternal-reduced_ninternal-1,dimension,temp_u[1,g],initial,ising_data)
  temp_x[g,] = simulate_ising_observations(current_x_latent,temp_u[2,g])
  total_sims <<- total_sims + (ninternal-reduced_ninternal)*dimension*dimension
}

for (h in 1:(num_imp_u/num_kept))
{
  for (g in 1:(num_kept))
  {
    current_x[(h-1)*num_kept+g,] = temp_x[g,]
  }
}

current_error = matrix(0,(num_imp_u))
for (g in 1:(num_imp_u))
{
  current_error[g] = abs(find_statistic(current_x[g,],edges)-ysum)
}
current_w = matrix(-log(num_imp_u),num_imp_u)

counter = 0
first = TRUE

current_epsilon = max(current_error)
eps_store = c(eps_store,current_epsilon)
write.matrix(eps_store,file=paste("epsilon",experiment_number,".txt",sep="_"))
write.matrix(current_u,file=paste("particles",experiment_number,counter,".txt",sep="_"))

while (current_epsilon>=epsilon)
{
  counter = counter + 1
  
  #current_epsilon = max(current_error)
  old_epsilon = current_epsilon
  resample_u = runif(num_imp_u,0,1)
  
  current_epsilon = bisection_u(c(0,current_epsilon),current_error,alpha,num_imp_u,resample_u,current_u)#epsilons[e]#
  
  for (g in 1:num_imp_u)
  {
    if (current_error[g]<current_epsilon)
    {
      current_w[g] = -log(num_imp_u)
    }
    else
    {
      current_w[g] = -Inf
    }
  }
  
  print("Epsilon:")
  print(current_epsilon)
  
  if (  (current_epsilon<epsilon) )
  {
    for (g in 1:num_imp_u)
    {
      if (current_error[g]<epsilon)
      {
        current_w[g] = -log(num_imp_u)
      }
      else
      {
        current_w[g] = -Inf
      }
    }
    
    unique_before_store = c(unique_before_store,length(unique(current_u[1,])))
    
    indices = stratified_resample_u(current_w,resample_u)
    current_x = current_x[indices,]
    current_reduced_x = current_reduced_x[indices,]
    current_u = current_u[,indices]
    current_error = current_error[indices]
    current_w = matrix(-log(num_imp_u),num_imp_u)
    
    #plot(current_u[1,],current_u[2,])
    
    print(total_sims)
    print(length(unique(current_u[1,])))
    
    print("Number:")
    print(number_accepted)
    
    #line = toString(number_accepted)
    #write(line,file="past_stage2.txt",append=TRUE)
    
    #line = toString(total_sims)
    #write(line,file="total_sims.txt",append=TRUE)
    
    #line = toString(current_epsilon)
    #write(line,file="epsilon.txt",append=TRUE)
    
    #line = toString(da_epsilon)
    #write(line,file="da_epsilon.txt",append=TRUE)
    
    #line = toString(length(unique(current_u[1,])))
    #write(line,file="unique.txt",append=TRUE)
    
    eps_store = c(eps_store,epsilon)
    unique_store = c(unique_store,length(unique(current_u[1,])))
    total_sims_store = c(total_sims_store,total_sims)
    
    write.matrix(unique_store,file=paste("unique",experiment_number,".txt",sep="_"))
    write.matrix(total_sims_store,file=paste("total_sims",experiment_number,".txt",sep="_"))
    write.matrix(eps_store,file=paste("epsilon",experiment_number,".txt",sep="_"))
    write.matrix(current_u,file=paste("particles",experiment_number,counter,".txt",sep="_"))

    break
  }
  
  unique_before_store = c(unique_before_store,length(unique(current_u[1,])))

  indices = stratified_resample_u(current_w,resample_u)
  
  current_x = current_x[indices,]
  current_reduced_x = current_reduced_x[indices,]
  current_u = current_u[,indices]
  current_error = current_error[indices]
  current_w = matrix(-log(num_imp_u),num_imp_u)
  
  print(length(unique(current_u[1,])))
  
  #plot(current_u[1,],current_u[2,])
  
  prop_cov = cov(t(current_u))
  
  eps_store = c(eps_store,current_epsilon)
  unique_store = c(unique_store,length(unique(current_u[1,])))
  total_sims_store = c(total_sims_store,total_sims)
  
  write.matrix(unique_before_store,file=paste("unique_before",experiment_number,".txt",sep="_"))
  write.matrix(unique_store,file=paste("unique",experiment_number,".txt",sep="_"))
  write.matrix(total_sims_store,file=paste("total_sims",experiment_number,".txt",sep="_"))
  write.matrix(eps_store,file=paste("epsilon",experiment_number,".txt",sep="_"))
  write.matrix(current_u,file=paste("particles",experiment_number,counter,".txt",sep="_"))
  
  all_logu = log(runif(num_imp_u))
  all_proposed_u = matrix(0,2,num_imp_u)
  all_logratio = matrix(0,num_imp_u)
  old_reduced_x = matrix(1,num_imp_u,reduced_dimension*reduced_dimension)
  old_reduced_error = matrix(Inf,num_imp_u)
  for (g in 1:num_imp_u)
  {
    all_proposed_u[,g] = rmvnorm(1,current_u[,g],prop_cov)
    all_logratio[g] = dnorm(all_proposed_u[1,g],0,prior_sd,log=TRUE) + dnorm(all_proposed_u[2,g],0,prior_sd,log=TRUE) - dnorm(current_u[1,g],0,prior_sd,log=TRUE) - dnorm(current_u[2,g],0,prior_sd,log=TRUE)
    old_reduced_x[g,] = current_reduced_x[g,]
    old_reduced_error[g] = abs(find_statistic(old_reduced_x[g,],reduced_edges)-reduced_ysum)
  }
  
  which_accept = which(all_logu<all_logratio)
  
  print("Num past prior")
  print(length(which_accept))
  
  num_past_prior_store = c(num_past_prior_store,length(which_accept))
  
  #line = toString(length(which_accept))
  #write(line,file="past_prior.txt",append=TRUE)
  
  reduced_proposed_x = matrix(1,num_imp_u,reduced_dimension*reduced_dimension)
  reduced_proposed_latent_x = matrix(1,num_imp_u,reduced_dimension*reduced_dimension)
  reduced_proposed_error = matrix(Inf,num_imp_u)
  
  if (length(which_accept)>num_kept)
  {
    for (g in 1:length(which_accept))
    {
      proposed_x_latent = simulate_ising_chess(1,reduced_ninternal-1,reduced_dimension,all_proposed_u[1,which_accept[g]],matrix(1,reduced_dimension*reduced_dimension),reduced_ising_data)
      reduced_proposed_latent_x[which_accept[g],] = proposed_x_latent
      reduced_proposed_x[which_accept[g],] = simulate_ising_observations(proposed_x_latent,all_proposed_u[2,which_accept[g]])
      total_sims <<- total_sims + (reduced_ninternal)*reduced_dimension*reduced_dimension
      reduced_proposed_error[which_accept[g]] = abs(find_statistic(reduced_proposed_x[which_accept[g],],reduced_edges)-reduced_ysum)
    }
    
    da_epsilon = max(max(reduced_proposed_error[which_accept]),max(old_reduced_error[which_accept]))
    current_score = da_score(da_epsilon,reduced_proposed_error[which_accept],old_reduced_error[which_accept],num_kept)
    
    if (current_score>0)
    {
      da_epsilon = da_bisection(c(0,da_epsilon),reduced_proposed_error[which_accept],old_reduced_error[which_accept],num_kept)
    }

    which_accept = which( (reduced_proposed_error<=da_epsilon) & (old_reduced_error<=da_epsilon) )
  }
  
  da_eps_store = c(da_eps_store,da_epsilon)
  
  print("Num past stage 1")
  print(length(which_accept))
  
  num_past_stage1_store = c(num_past_stage1_store,length(which_accept))
  
  #line = toString(length(which_accept))
  #write(line,file="past_stage1.txt",append=TRUE)
  
  number_accepted = 0
  
  for (g in 1:length(which_accept))
  {
    initial = matrix(repmat(matrix(reduced_proposed_latent_x[which_accept[g],],reduced_dimension,reduced_dimension),dimension/reduced_dimension,dimension/reduced_dimension),dimension*dimension)
    proposed_x_latent = simulate_ising_chess(1,ninternal-reduced_ninternal-1,dimension,all_proposed_u[1,which_accept[g]],initial,ising_data)
    proposed_x = simulate_ising_observations(proposed_x_latent,all_proposed_u[2,which_accept[g]])
    total_sims <<- total_sims + (ninternal-reduced_ninternal)*dimension*dimension
    proposed_error = abs(find_statistic(proposed_x,edges)-ysum)
    if (proposed_error<current_epsilon)
    {
      current_u[,which_accept[g]] = all_proposed_u[,which_accept[g]]
      current_x[which_accept[g],] = proposed_x
      current_reduced_x[which_accept[g],] = reduced_proposed_x[which_accept[g],]
      current_error[which_accept[g]] = proposed_error
      number_accepted = number_accepted + 1
    }
  }
  
  print("Number:")
  print(number_accepted)
  
  num_accepted_store = c(num_accepted_store,number_accepted)
  
  write.matrix(da_eps_store,file=paste("da_epsilon",experiment_number,".txt",sep="_"))
  write.matrix(num_past_prior_store,file=paste("num_past_prior",experiment_number,".txt",sep="_"))
  write.matrix(num_past_stage1_store,file=paste("num_past_stage1",experiment_number,".txt",sep="_"))
  write.matrix(num_accepted_store,file=paste("num_accepted",experiment_number,".txt",sep="_"))
  
  #line = toString(number_accepted)
  #write(line,file="past_stage2.txt",append=TRUE)
  
  #line = toString(total_sims)
  #write(line,file="total_sims.txt",append=TRUE)
  
  #line = toString(current_epsilon)
  #write(line,file="epsilon.txt",append=TRUE)
  
  #line = toString(da_epsilon)
  #write(line,file="da_epsilon.txt",append=TRUE)
  
  #line = toString(length(unique(current_u[1,])))
  #write(line,file="unique.txt",append=TRUE)
}
